/* *********************************************************************** */ 
/*                              ST.H v2.5                                  */
/*                   1989-1990 Jean-Francois Giorgi                        */
/* *********************************************************************** */ 
/*       
  This is the include file for the Soudtracker V2.5 song file format.
  

  format of ST song file:

  | NAME            | : 20 bytes
  | 1st instrument  | : 30 bytes
   ...
  | 31th instrument | : 30 bytes
  | length          | : 1 byte   - length of the score
  | 0x78            | : 1 byte   - ?
  | patpos[128]     | : 128 bytes 
  | "M.K."          | : 4 bytes  - ?
  | 1st pattern     | : 64 events
   .....
  | lth pattern    |  : 64 events 


patpos[n] = # of the pattern to play at position n

l = max(patpos[n]) for n = 0 to length

 instrument     = | name | midsize | volume | repeat | midreapeatlenght |
 sizes in bytes :    22        2         2       2            2

   midsize          = half of the size of the sample
   repeat           = position/2 of the beginning of the repeat part of the sample
   midreapeatlenght = half of the size of the repeat part (1 if none)

 event = | note 1 | note 2 | note 3 | note 4 | 
 ie  one note per hardware channel                      

 note          = | upperins | tone | ins | control | data | 
 sizes in bits :      4        12     4       4       8     

 upperins * 16 + ins = instrument number ( 0 to 31)
 tone                = hardware period
 control = 0x0 -> normal or arpeggio if data != 0
           0x1 -> portamento up
           0x2 -> portamento down
           0x3 -> portamento
           0x4 -> vibrato
           0xa -> slide volume
           0xb -> position jump (goto)
           0xc -> set volume
           0xd -> pattern break (ie skip rest of the pattern)
           0xe -> filter (red light)
           0xf -> set speed

 data = value for control 
 
Comments on some controls (from D.O.C SoundTracker v2.0 'track.doc'):
-------------------------------------------------------------------------------

examples are given in the form :

               C#1  03C23
               /    / |\|
              /    /  | Data
             /    /  Command
            /   Inst#
         Note to
         be played.


Arpeggio:  Arpeggio is used for simulating accords. It does this by
---------  rapidly changing the notes while playing an instrument.
           Arpeggios are selected whenever the info byte has a non-zero
           value and no other commands are selected.
           When arpeggios are used, SoundTracker looks five times during
           the play of one note, at the information contained in the info
           byte. Each nibble (4 bits) in this byte defines
           how many notes SoundTracker must add to the the note currently
           playing. In each of the five passes SoundTracker does the
           following with the information contained in the info byte :
       
           1 - Add the first nibble to the current note.

           2 - Add the second nibble to the current note.

           3 - Play the current note.

           4 - Add the first nibble to the current note.

           5 - Add the second nibble to the current note.

 
Portamento:  Portamento is used for 'sliding' of the instrument.
-----------  If portamento is in use, the information contained in
             the info byte will be added to (or subtracted from) the period
             of the note playing in each of the five passes.

   Example:  D#2 03208  Slides down $08
             --- 00208  Slides down $08
             --- 00108  Slides up $08
             --- 00210  Slides down $10 (=16)
             etc.

   
Position jump:
--------------
         The position jump command will do the same like the pattern break
         command, but the info byte gives the playroutine the no. of the
         position.

Example: E-2 03B04 - Break the pattern and jump to position 4
         --- 00000
         --- 00B06 - Break the pattern and jump to position 6


Volume:  One of the enhancements that the V2.0 version provides is the
-------  ability to change the volume at any time during play.
         When command $0C (=12) is selected, the volume is set to the value
         contained in the info byte. The volume will remain at this level
         until a next SAMPLE is played. At this time, the volume is
         reset to the default volume level of that sample.

Example: C#1 01C22 - Set volume to $22 (=34) and use instrument # 1 to
                    play the C#1 note.
         D-2 00000 - Use instrument # 1 to play the D-2 note.
                    (Volume level remains at $22 (=34))
         B-1 01000 - Use instrument # 1 to play the B-1 note and
                    set volume to default value for that
                    instrument.

Slide Volume: Smoothly slide the volume up or down.
------------

Example: A#3 01a05 - slide the volume down 5 units per tick.
         --- 00a50 - slide the volume up 5 units per tick.

Pattern break:
--------------
         Pattern break is used to create shorter patterns than 64 notes.
         The note with the $0D command will be played and after that
         the next pattern in the position-table will be played.

Example: A#3 01D00 - At this point the pattern interrupts.
         --- 00D00 - Look upper line.
         C-1 00D30 - Look upper line.


Filter:  This Soundtracker version allows you to turn the lo-pass filter
-------  on/off.

Example: G#2 01E01 - Turns the filter and the power-led off.
         --- 00000
         E-1 01E00 - Turns the filter and the power-led on.

Warning to all Amiga 1000 owners :
Don't play with the power-led because some A500 and A2000 owners will
not be happy when they hear your song.


Speed:   This function allows changing the speed during play.
------   You can only enter a speed between 01 and 0F.
         Setting the info byte to zero doesn't effect anything.
         The normal speed is 6.

Example: D-2 04F05 - Set speed to 5
         --- 00000
         E-1 00F00 - Nothing happens to speed.
         --- 00FC3 - Set speed to 3
         G-2 02F06 - Set normal speed

playing :

playing is handled by a vertical blanking interrupt (VBI) routine.
the VBI frequency is 1/60 sec (tick).
A counter is initialized with the value of the speed (1 to 15).
At each VBI this counter is decremented.
When it reaches zero, the routine process the next line.
So the duration of a line is : speed/60 secondes.
At each VBI arpeggio,portamento,volume slide are also processed.

-------------------------------------------------------------------------------
*/
 
#ifndef ST_H 
#define ST_H 
 
#include <exec/types.h>

#define NAME_SIZE 20
 
typedef char ST_ScoreName[NAME_SIZE];

#define STI_NAME_SIZE 22
#define DEFAULT_MDRL 1L /* Default midreapeatlenght */

typedef struct {
  char sti_name[STI_NAME_SIZE];  /* name (st-01 relatif or complete path) */
  UWORD sti_midsize;        /* half size of the corresponding file   */
  UWORD sti_volume;        /* instrument volume: 0 to MAX_VOLUME    */
  UWORD sti_repeat;           /* repeat starting position */
  UWORD sti_midrepeatlength; /* repeat half size         */
} ST_Instrument;

#define MAX_LENGTH 128 /* max num of positions */

/* that's the byte and the long i don't understand what there are for */
#define SH_MARK 0x78
#define IDENT_MARK 0x4D2E4B2EL /* = "M.K." */

typedef struct { 
  UBYTE  stsc_length;               /* Length of the score    */
  UBYTE  stsc_mark;                /* should be set to 0x78  */
  UBYTE  stsc_patpos[MAX_LENGTH]; /* #pattern at pos        */ 
  ULONG  stsc_ident;             /* ident mark             */
} ST_ScoreHeader;

typedef struct {
  unsigned
    stn_upperins:4,     /* instrument part 1 (LSB 0-3)*/
    stn_period  :12;     /* note period  0x1FC = A2 */
  unsigned
    stn_ins  : 4, /* instrument part 2 (MSB 4-8) */
    stn_ctrl : 4,  /* Control      */
    stn_data : 8;   /* Control data */
} ST_Note;

/* stn_ctrl may be : */
 
#define STC_NORMAL   0x0 /* normal or arpeggio if stn_data != 0 */
#define STC_PORTA_UP 0x1 /* portamento up                       */
#define STC_PORTA_DW 0x2 /* portamento down                     */
#define STC_PORTA    0x3 /* portamento (noisetracker1.2 spec)   */
#define STC_VIBRATO  0x4 /* vibrato    (noisetracker1.2 spec)   */
#define STC_SLIDEVOL 0xa /* slide volume                        */
#define STC_JUMP     0xb /* position jump                       */
#define STC_VOLUME   0xc /* set volume                          */
#define STC_BREAK    0xd /* pattern break                       */
#define STC_FILTER   0xe /* filter (red light)                  */
#define STC_SPEED    0xf /* set speed                           */

#define MAX_VOLUME   0x40 /* max supported volume (hardware)*/

#define NUM_VOICES 4      /* number of hardware voices */

typedef ST_Note ST_Event[NUM_VOICES]; /* one Event = four notes */

#define PAT_LEN 64     /* 64 Events per pattern */
#define MAX_PAT 64    /* max num of pattern */

typedef ST_Event ST_Pattern[PAT_LEN]; 

#ifdef STV1
#define MAX_INST 15         /* max 15 instruments per score */
#else
#define MAX_INST 31         /* max 31 instruments per score */
#endif

typedef struct {
  ST_ScoreName   sts_name;             /* The name of the score */
  ST_Instrument  sts_instr[MAX_INST]; /* array of instruments  */
  /* ST_ScoreHeader sts_sh; */            /* header of the score   */
  UBYTE  sts_length;               /* Length of the score    */
  UBYTE  sts_mark;                /* should be set to 0x78  */
  UBYTE  sts_patpos[MAX_LENGTH]; /* #pattern at pos        */
  ULONG  sts_ident;             /* ident mark             */
} ST_Score;
  
/* total song size in bytes given the number of patterns (np)*/
#define SZ_ST_SCORE ((LONG)(sizeof(ST_Score)))

#endif /* ST_H */
